#include "stdafx.h"
#include <vector>
#include "EikonDesktopEventListener.h"
#include <atlstr.h>

EikonDesktopEventListener::EikonDesktopEventListener(CEikonDesktopDataAPI* p)
{
	m_eikonDesktopAPI = p;
}

EikonDesktopEventListener::~EikonDesktopEventListener(void)
{
}

CString EikonDesktopEventListener::GetPlatformStatusText(EEikonStatus a_eStatus)
{
	switch (a_eStatus)
	{
	case Connected:
		return _T("Connected");
	case Disconnected:
		return _T("Disconnected");
	case LocalMode:
		return _T("LocalMode");
	case Offline:
		return _T("Offline");
	}
	return _T("Unknown");
}

CString EikonDesktopEventListener::GetInstrumentStatusDescription(long a_itemStatus)
{
	CString l_description;

	switch(a_itemStatus)
	{
	case RT_ITEM_OK:
		l_description = _T("RT_ITEM_OK");
		break;
	case RT_ITEM_INVALID:
		l_description = _T("RT_ITEM_INVALID");
		break;
	case RT_ITEM_UNKNOWN:
		l_description = _T("RT_ITEM_UNKNOWN");
		break;
	case RT_ITEM_STALE:
		l_description = _T("RT_ITEM_STALE");
		break;
	case RT_ITEM_DELAYED:
		l_description = _T("RT_ITEM_DELAYED");
		break;
	case RT_ITEM_NOT_PERMISSIONED:
		l_description = _T("RT_ITEM_NOT_PERMISSIONED");
		break;
	default:
		l_description = _T("Unknown item status!!!");
		break;
	}
	
	return l_description;
}


CString EikonDesktopEventListener::GetFieldStatusDescription(long a_fieldStatus)
{
	CString l_description = _T("");

	switch(a_fieldStatus)
	{
	case RT_FIELD_OK:
		l_description = _T("RT_FIELD_OK");
		break;
	case RT_FIELD_INVALID:
		l_description = _T("RT_FIELD_INVALID");
		break;
	case RT_FIELD_UNKNOWN:
		l_description = _T("RT_FIELD_UNKNOWN");
		break;
	case RT_FIELD_UNDEFINED:
		l_description = _T("RT_FIELD_UNDEFINED");
		break;
	default:
		l_description = _T("");
		break;
	}

	return l_description;
}

void EikonDesktopEventListener::GetFieldInformation(CString a_itemName, 
									  std::vector<CString> &a_fieldNames, 
									  std::vector<CString> &a_fieldValues, 
									  std::vector<CString> &a_fieldStatuses )
{
	int l_fieldColumnView = RT_FCV_VALUE|RT_FCV_STATUS;
	_variant_t l_listFields;
	try
	{
		l_listFields.Attach(m_adxRtList.get_ListFields( a_itemName, RT_FRV_UPDATED, (RT_FieldColumnView)l_fieldColumnView));
	}
	catch (_com_error& l_exception)
	{
		_bstr_t l_errorMessage = l_exception.Description();
		wprintf_s(L"%s\n", l_errorMessage);
	}

	if ( (l_listFields.vt == VT_EMPTY) || (l_listFields.vt != (VT_VARIANT | VT_ARRAY) ))
	{
		return;
	}

	SAFEARRAY *l_sa = l_listFields.parray;
	LONG l_fieldStart = ((*l_sa).rgsabound[1]).lLbound;
	LONG l_fieldEnd = l_fieldStart + ((*l_sa).rgsabound[1]).cElements;
	LONG l_colStart = ((*l_sa).rgsabound[0]).lLbound;
	for ( LONG j = l_fieldStart; j < l_fieldEnd; ++j )
	{
		LONG l_indices[2];

		l_indices[0] = j;
		l_indices[1] = l_colStart;	// Column of ItemName
		_variant_t l_varFieldName;
		SafeArrayGetElement(l_sa, l_indices, &l_varFieldName);

		l_indices[0] = j;
		l_indices[1] = l_colStart + 1; // Column of ItemStatus
		_variant_t l_varFieldValue;
		SafeArrayGetElement(l_sa, l_indices, &l_varFieldValue);

		_variant_t l_varFieldStatus;
		
		l_indices[0] = j;
		l_indices[1] = 2;
		SafeArrayGetElement(l_sa, l_indices, &l_varFieldStatus);

		a_fieldNames.push_back( CString(l_varFieldName) );
		a_fieldValues.push_back( CString(l_varFieldValue) );
		a_fieldStatuses.push_back( 
			GetFieldStatusDescription(l_varFieldStatus.lVal )
		);
	}
}

void EikonDesktopEventListener::OnStatusChanged(EEikonStatus a_eStatus)
{
	wprintf_s(L"%s\n", GetPlatformStatusText(a_eStatus));
	if (a_eStatus == Connected)
	{
		wprintf_s(L"Initializing AdxRtList...\n");
		m_isAdxRtListAvailable = S_OK == m_adxRtList.Initialize(this, m_eikonDesktopAPI);
		wprintf_s(L"Subscribing to realtime data...\n");
		m_adxRtList.put_Source(L"IDN");
		m_adxRtList.RegisterItems(_variant_t("EUR="), _variant_t("BID"));
		m_adxRtList.StartUpdates(RT_MODE_ONUPDATE);
		wprintf_s(L"\n");
	}
}

void EikonDesktopEventListener::OnUpdate(BSTR a_itemName, VARIANT a_userTag, enum RT_ItemStatus a_itemStatus)
{
	CString l_itemName(a_itemName);
	CString l_itemStatusDescription = GetInstrumentStatusDescription(a_itemStatus);
	// Display the received information
	std::vector<CString> l_fieldNames;
	std::vector<CString> l_fieldValues;
	std::vector<CString> l_fieldStatuses;
	GetFieldInformation(l_itemName, l_fieldNames, l_fieldValues, l_fieldStatuses);
	for ( size_t j = 0; j < l_fieldNames.size(); ++j )
	{
		CString l_displayMessage;
		l_displayMessage.Format(
			_T("[%s][%s]   [%s][%s][%s]"),
			l_itemStatusDescription,
			l_fieldStatuses[j],
			l_itemName, 
			l_fieldNames[j],
			l_fieldValues[j]
		);
		wprintf_s(L"%s\n", l_displayMessage);
	}
}

void EikonDesktopEventListener::OnImage(RT_DataStatus a_dataStatus)
{
}

void EikonDesktopEventListener::OnTime()
{
}

void EikonDesktopEventListener::OnStatusChange(RT_ListStatus a_listStatus, RT_SourceStatus a_sourceStatus, RT_RunMode a_runMode)
{
}

